from discord.errors import Forbidden
from discord import AuditLogAction
from datetime import datetime
from asyncio import sleep

class InviteTracker():
    def __init__(self, bot):
        self.bot = bot
        self._cache = {}
        self.add_listeners()
    
    def add_listeners(self):
        self.bot.add_listener(self.cache_invites, "on_ready")
        self.bot.add_listener(self.update_invite_cache, "on_invite_create")
        self.bot.add_listener(self.remove_invite_cache, "on_invite_delete")
        self.bot.add_listener(self.add_guild_cache, "on_guild_join")
        self.bot.add_listener(self.remove_guild_cache, "on_guild_remove")
    
    async def cache_invites(self):
        for guild in self.bot.guilds:
            try:
                self._cache[guild.id] = {}
                for invite in await guild.invites():
                    self._cache[guild.id][invite.code] = invite
            except Forbidden:
                continue
    
    async def update_invite_cache(self, invite):
        if invite.guild.id not in self._cache.keys():
            self._cache[invite.guild.id] = {}
        self._cache[invite.guild.id][invite.code] = invite
    
    async def remove_invite_cache(self, invite):
        if invite.guild.id not in self._cache.keys():
            return
        ref_invite = self._cache[invite.guild.id][invite.code]
        if (ref_invite.created_at.timestamp()+ref_invite.max_age > datetime.utcnow().timestamp() or ref_invite.max_age == 0) and ref_invite.max_uses > 0 and ref_invite.uses == ref_invite.max_uses-1:
            try:
                async for entry in invite.guild.audit_logs(limit=1, action=AuditLogAction.invite_delete):
                    if entry.target.code != invite.code:
                        self._cache[invite.guild.id][ref_invite.code].revoked = True
                        return
                else:
                    self._cache[invite.guild.id][ref_invite.code].revoked = True
                    return
            except Forbidden:
                self._cache[invite.guild.id][ref_invite.code].revoked = True
                return
        else:
            self._cache[invite.guild.id].pop(invite.code)
    
    async def add_guild_cache(self, guild):
        self._cache[guild.id] = {}
        for invite in await guild.invites():
            self._cache[guild.id][invite.code] = invite
    
    async def remove_guild_cache(self, guild):
        try:
            self._cache.pop(guild.id)
        except KeyError:
            return
    
    async def fetch_inviter(self, member):
        await sleep(self.bot.latency)
        for new_invite in await member.guild.invites():
            for cached_invite in self._cache[member.guild.id].values():
                if new_invite.code == cached_invite.code and new_invite.uses - cached_invite.uses == 1 or cached_invite.revoked:
                    if cached_invite.revoked:
                        self._cache[member.guild.id].pop(cached_invite.code)
                    elif new_invite.inviter == cached_invite.inviter:
                        self._cache[member.guild.id][cached_invite.code] = new_invite
                    else:
                        self._cache[member.guild.id][cached_invite.code].uses += 1
                    return cached_invite.inviter


from DiscordUtils import Pagination
from DiscordUtils.InviteTracker import InviteTracker
from DiscordUtils.Music import Music

__title__ = "DisUtils"
__version__ = "1.3.5"
__author__ = "DTS"
__license__ = "MIT"



import discord
import asyncio
from discord.ext import commands

class AutoEmbedPaginator(object):
    def __init__(self, ctx, **kwargs):
        self.embeds = None
        self.ctx = ctx
        self.bot = ctx.bot
        self.current_page = 0
        self.auto_footer = kwargs.get("auto_footer", False)
        self.remove_reactions = kwargs.get("remove_reactions", False)
        self.control_emojis = ('⏮️', '⏪', '🔐', '⏩', '⏭️')
        self.timeout = int(kwargs.get("timeout", 60))
    async def run(self, embeds, send_to=None):
        if not send_to:
            send_to = self.ctx
        wait_for = self.ctx.author if send_to == self.ctx else send_to
        if not self.embeds:
            self.embeds = embeds
        if self.auto_footer:
            self.embeds[0].set_footer(text=f'({self.current_page+1}/{len(self.embeds)})')
        msg = await send_to.send(embed=self.embeds[0])
        for emoji in self.control_emojis:
            try:
                await msg.add_reaction(emoji)
            except:
                pass
        msg = await msg.channel.fetch_message(msg.id)
        def check(reaction, user):
            return user == wait_for and reaction.message.id == msg.id and str(reaction.emoji) in self.control_emojis
        while True:
            if self.timeout > 0:
                try:
                    reaction, user = await self.bot.wait_for("reaction_add",check=check, timeout=self.timeout)
                except asyncio.TimeoutError:
                    self.current_page = 0
                    for reaction in msg.reactions:
                        if reaction.message.author.id == self.bot.user.id:
                            try:
                                await msg.remove_reaction(str(reaction.emoji), reaction.message.author)
                            except:
                                pass
                    return msg
                    break
            else:
                reaction, user = await self.bot.wait_for("reaction_add",check=check)
            if str(reaction.emoji) == self.control_emojis[0]:
                self.current_page = 0
                if self.remove_reactions:
                    try:
                        await msg.remove_reaction(str(reaction.emoji), user)
                    except:
                        pass
                if self.auto_footer:
                    self.embeds[0].set_footer(text=f'({self.current_page+1}/{len(self.embeds)})')
                await msg.edit(embed=self.embeds[0])
            elif str(reaction.emoji) == self.control_emojis[1]:
                self.current_page = self.current_page-1
                self.current_page = 0 if self.current_page<0 else self.current_page
                if self.remove_reactions:
                    try:
                        await msg.remove_reaction(str(reaction.emoji), user)
                    except:
                        pass
                if self.auto_footer:
                    self.embeds[self.current_page].set_footer(text=f'({self.current_page+1}/{len(self.embeds)})')
                await msg.edit(embed=self.embeds[self.current_page])
            elif str(reaction.emoji) == self.control_emojis[2]:
                self.current_page = 0
                for reaction in msg.reactions:
                    try:
                        if reaction.message.author.id == self.bot.user.id:
                            await msg.remove_reaction(str(reaction.emoji), reaction.message.author)
                    except:
                        pass
                return msg
                break
            elif str(reaction.emoji) == self.control_emojis[3]:
                self.current_page = self.current_page + 1
                self.current_page = len(self.embeds)-1 if self.current_page > len(self.embeds)-1 else self.current_page
                if self.remove_reactions:
                    try:
                        await msg.remove_reaction(str(reaction.emoji), user)
                    except:
                        pass
                if self.auto_footer:
                    self.embeds[self.current_page].set_footer(text=f'({self.current_page+1}/{len(self.embeds)})')
                await msg.edit(embed=self.embeds[self.current_page])
            elif str(reaction.emoji) == self.control_emojis[4]:
                self.current_page = len(self.embeds)-1
                if self.remove_reactions:
                    try:
                        await msg.remove_reaction(str(reaction.emoji), user)
                    except:
                        pass
                if self.auto_footer:
                    self.embeds[len(self.embeds)-1].set_footer(text=f'({self.current_page+1}/{len(self.embeds)})')
                await msg.edit(embed=self.embeds[len(self.embeds)-1])
                
class CustomEmbedPaginator(object):
    def __init__(self, ctx, **kwargs):
        self.embeds = None
        self.ctx = ctx
        self.bot = ctx.bot
        self.timeout = int(kwargs.get("timeout",60))
        self.current_page = 0
        self.control_emojis = []
        self.control_commands = []
        self.auto_footer = kwargs.get("auto_footer", False)
        self.remove_reactions = kwargs.get("remove_reactions", False)
    def add_reaction(self, emoji, command):
        self.control_emojis.append(emoji)
        self.control_commands.append(command)
    def insert_reaction(self, index, emoji, command):
        self.control_emojis.insert(index, emoji)
        self.control_commands.insert(index, command)
    def remove_reaction(self, emoji):
        if emoji in self.control_emojis:
            index = self.control_emojis.index(emoji)
            self.control_emojis.remove(emoji)
            self.control_commands.pop(index)
    def remove_reaction_at(self, index):
        if index > len(self.control_emojis)-1:
            index = len(self.control_emojis)-1
        elif index < 0:
            index = 0
        try:
            self.control_emojis.pop(index)
            self.control_commands.pop(index)
        except:
            pass
    def clear_reactions(self):
        self.control_emojis = []
        self.control_commands = []
    async def run(self, embeds, send_to=None):
        self.embeds = embeds
        if not send_to:
            send_to = self.ctx
        wait_for = self.ctx.author if send_to == self.ctx else send_to
        if self.auto_footer:
            self.embeds[0].set_footer(text=f'({self.current_page+1}/{len(self.embeds)})')
        msg = await send_to.send(embed=self.embeds[0])
        for emoji in self.control_emojis:
            try:
                await msg.add_reaction(emoji)
            except:
                pass
        msg = await msg.channel.fetch_message(msg.id)
        def check(reaction, user):
            return user == wait_for and reaction.message.id == msg.id and str(reaction.emoji) in self.control_emojis
        while True:
            if self.timeout > 0:
                try:
                    reaction, user = await self.bot.wait_for("reaction_add",check=check, timeout=self.timeout)
                except asyncio.TimeoutError:
                    for reaction in msg.reactions:
                        if reaction.message.author.id == self.bot.user.id and self.remove_reactions:
                                try:
                                    await msg.remove_reaction(str(reaction.emoji), reaction.message.author)
                                except:
                                    pass
                    self.current_page = 0
                    return msg
                    break
            else:
                reaction, user = await self.bot.wait_for("reaction_add",check=check)
            for emoji in self.control_emojis:
                if emoji == str(reaction.emoji):
                    index = self.control_emojis.index(emoji)
                    cmd = self.control_commands[index]
                    if cmd.lower() == "first":
                        self.current_page = 0
                        if self.remove_reactions:
                            try:
                                await msg.remove_reaction(str(reaction.emoji), user)
                            except:
                                pass
                        if self.auto_footer:
                            self.embeds[0].set_footer(text=f'({self.current_page+1}/{len(self.embeds)})')
                        await msg.edit(embed=self.embeds[0])
                    elif cmd.lower() == "last":
                        self.current_page = len(self.embeds)-1
                        if self.remove_reactions:
                            try:
                                await msg.remove_reaction(str(reaction.emoji), user)
                            except:
                                pass
                        if self.auto_footer:
                            self.embeds[len(self.embeds)-1].set_footer(text=f'({self.current_page+1}/{len(self.embeds)})')
                        await msg.edit(embed=self.embeds[len(self.embeds)-1])
                    elif cmd.lower() == "next":
                        self.current_page += 1
                        self.current_page = len(self.embeds)-1 if self.current_page > len(self.embeds)-1 else self.current_page
                        if self.remove_reactions:
                            try:
                                await msg.remove_reaction(str(reaction.emoji), user)
                            except:
                                pass
                        if self.auto_footer:
                            self.embeds[self.current_page].set_footer(text=f'({self.current_page+1}/{len(self.embeds)})')
                        await msg.edit(embed=self.embeds[self.current_page])
                    elif cmd.lower() == "back":
                        self.current_page = self.current_page-1
                        self.current_page = 0 if self.current_page<0 else self.current_page
                        if self.remove_reactions:
                            try:
                                await msg.remove_reaction(str(reaction.emoji), user)
                            except:
                                pass
                        if self.auto_footer:
                            self.embeds[self.current_page].set_footer(text=f'({self.current_page+1}/{len(self.embeds)})')
                        await msg.edit(embed=self.embeds[self.current_page])
                    elif cmd.lower() == "delete":
                        self.current_page = 0
                        await msg.delete()
                        return msg
                        break
                    elif cmd.lower() == "clear" or cmd.lower() == "lock":
                        self.current_page = 0
                        for reaction in msg.reactions:
                            try:
                                await msg.clear_reactions()
                                break
                            except discord.Forbidden or discord.HTTPException:
                                if reaction.message.author.id == self.bot.user.id:
                                    try:
                                        await msg.remove_reaction(str(reaction.emoji), reaction.message.author)
                                    except:
                                        pass
                        return msg
                        break
                    elif cmd.startswith("page"):
                        shit = cmd.split()
                        pg = int(shit[1])
                        self.current_page = pg
                        if pg > len(embeds)-1:
                            pg = len(embeds)-1
                        if pg < 0:
                            pg = 0
                        if self.remove_reactions:
                            try:
                                await msg.remove_reaction(str(reaction.emoji), user)
                            except:
                                pass
                        if self.auto_footer:
                            self.embeds[self.current_page].set_footer(text=f'({pg+1}/{len(self.embeds)})')
                        await msg.edit(embed=self.embeds[pg])
                    elif cmd.startswith("remove"):
                        things = cmd.split()
                        things.pop(0)
                        something = things[0]
                        if something.isdigit():
                            index = int(something)
                            index = len(self.control_emojis)-1 if index > len(self.control_emojis)-1 else index
                            index = 0 if index < 0 else index
                            emoji = self.control_emojis[index]
                            if self.remove_reactions:
                                try:
                                    await msg.remove_reaction(str(reaction.emoji), user)
                                except:
                                    pass
                            try:
                                await msg.remove_reaction(emoji, self.bot.user)
                            except:
                                pass
                            self.control_emojis.pop(index)
                            self.control_commands.pop(index)
                        else:
                            emoji = something
                            if emoji in self.control_emojis:
                                if self.remove_reactions:
                                    try:
                                        await msg.remove_reaction(str(reaction.emoji), user)
                                    except:
                                        pass
                                try:
                                    await msg.remove_reaction(emoji, self.bot.user)
                                except:
                                    pass
                                index = self.control_emojis.index(emoji)
                                self.control_emojis.remove(emoji)
                                self.control_commands.pop(index)



import asyncio
import aiohttp
import re
try:
    import youtube_dl
    import discord
    has_voice = True
except ImportError:
    has_voice = False

if has_voice:
    youtube_dl.utils.bug_reports_message = lambda: ''
    ydl = youtube_dl.YoutubeDL({"format": "bestaudio/best", "restrictfilenames": True, "noplaylist": True, "nocheckcertificate": True, "ignoreerrors": True, "logtostderr": False, "quiet": True, "no_warnings": True, "source_address": "0.0.0.0"})

class EmptyQueue(Exception):
    """Cannot skip because queue is empty"""
    
class NotConnectedToVoice(Exception):
    """Cannot create the player because bot is not connected to voice"""
    
class NotPlaying(Exception):
    """Cannot <do something> because nothing is being played"""
    
async def ytbettersearch(query):
    url = f"https://www.youtube.com/results?search_query={query}"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as resp:
            html = await resp.text()
    index = html.find('watch?v')
    url = ""
    while True:
        char = html[index]
        if char == '"':
            break
        url += char
        index += 1
    url = f"https://www.youtube.com/{url}"
    return url

async def get_video_data(url, search, bettersearch, loop):
    if not has_voice:
        raise RuntimeError("DiscordUtils[voice] install needed in order to use voice")

    if not search and not bettersearch:
        data = await loop.run_in_executor(None, lambda: ydl.extract_info(url, download=False))
        source = data["url"]
        url = "https://www.youtube.com/watch?v="+data["id"]
        title = data["title"]
        description = data["description"]
        likes = data["like_count"]
        views = data["view_count"]
        duration = data["duration"]
        thumbnail = data["thumbnail"]
        channel = data["uploader"]
        channel_url = data["uploader_url"]
        return Song(source, url, title, description, views, duration, thumbnail, channel, channel_url, False)
    else:
        if bettersearch:
            url = await ytbettersearch(url)
            data = await loop.run_in_executor(None, lambda: ydl.extract_info(url, download=False))
            source = data["url"]
            url = "https://www.youtube.com/watch?v="+data["id"]
            title = data["title"]
            description = data["description"]
            likes = data["like_count"]
            views = data["view_count"]
            duration = data["duration"]
            thumbnail = data["thumbnail"]
            channel = data["uploader"]
            channel_url = data["uploader_url"]
            return Song(source, url, title, description, views, duration, thumbnail, channel, channel_url, False)
        elif search:
            ytdl = youtube_dl.YoutubeDL({"format": "bestaudio/best", "restrictfilenames": True, "noplaylist": True, "nocheckcertificate": True, "ignoreerrors": True, "logtostderr": False, "quiet": True, "no_warnings": True, "default_search": "auto", "source_address": "0.0.0.0"})
            data = await loop.run_in_executor(None, lambda: ytdl.extract_info(url, download=False))
            try:
                data = data["entries"][0]
            except KeyError or TypeError:
                pass
            del ytdl
            source = data["url"]
            url = "https://www.youtube.com/watch?v="+data["id"]
            title = data["title"]
            description = data["description"]
            likes = data["like_count"]
            views = data["view_count"]
            duration = data["duration"]
            thumbnail = data["thumbnail"]
            channel = data["uploader"]
            channel_url = data["uploader_url"]
            return Song(source, url, title, description, views, duration, thumbnail, channel, channel_url, False)
        
def check_queue(ctx, opts, music, after, on_play, loop):
    if not has_voice:
        raise RuntimeError("DiscordUtils[voice] install needed in order to use voice")

    try:
        song = music.queue[ctx.guild.id][0]
    except IndexError:
        return
    if not song.is_looping:
        try:
            music.queue[ctx.guild.id].pop(0)
        except IndexError:
            return
        if len(music.queue[ctx.guild.id]) > 0:
            source = discord.PCMVolumeTransformer(discord.FFmpegPCMAudio(music.queue[ctx.guild.id][0].source, **opts))
            ctx.voice_client.play(source, after=lambda error: after(ctx, opts, music, after, on_play, loop))
            song = music.queue[ctx.guild.id][0]
            if on_play:
                loop.create_task(on_play(ctx, song))
    else:
        source = discord.PCMVolumeTransformer(discord.FFmpegPCMAudio(music.queue[ctx.guild.id][0].source, **opts))
        ctx.voice_client.play(source, after=lambda error: after(ctx, opts, music, after, on_play, loop))
        song = music.queue[ctx.guild.id][0]
        if on_play:
            loop.create_task(on_play(ctx, song))

class Music(object):
    def __init__(self):
        if not has_voice:
            raise RuntimeError("DiscordUtils[voice] install needed in order to use voice")

        self.queue = {}
        self.players = []

    def create_player(self, ctx, **kwargs):
        if not ctx.voice_client:
            raise NotConnectedToVoice("Cannot create the player because bot is not connected to voice")
        player = MusicPlayer(ctx, self, **kwargs)
        self.players.append(player)
        return player
        
    def get_player(self, **kwargs):
        guild = kwargs.get("guild_id")
        channel = kwargs.get("channel_id")
        for player in self.players:
            if guild and channel and player.ctx.guild.id == guild and player.voice.channel.id == channel:
                return player
            elif not guild and channel and player.voice.channel.id == channel:
                return player
            elif not channel and guild and player.ctx.guild.id == guild:
                return player
        else:
            return None

class MusicPlayer(object):
    def __init__(self, ctx, music, **kwargs):
        if not has_voice:
            raise RuntimeError("DiscordUtils[voice] install needed in order to use voice")

        self.ctx = ctx
        self.voice = ctx.voice_client
        self.loop = ctx.bot.loop
        self.music = music
        if self.ctx.guild.id not in self.music.queue.keys():
            self.music.queue[self.ctx.guild.id] = []
        self.after_func = check_queue
        self.on_play_func = self.on_queue_func = self.on_skip_func = self.on_stop_func = self.on_pause_func = self.on_resume_func = self.on_loop_toggle_func = self.on_volume_change_func = self.on_remove_from_queue_func = None
        ffmpeg_error = kwargs.get("ffmpeg_error_betterfix", kwargs.get("ffmpeg_error_fix"))
        if ffmpeg_error and "ffmpeg_error_betterfix" in kwargs.keys():
            self.ffmpeg_opts = {"options": "-vn -loglevel quiet -hide_banner -nostats", "before_options": "-reconnect 1 -reconnect_streamed 1 -reconnect_delay_max 0 -nostdin"}
        elif ffmpeg_error:
            self.ffmpeg_opts = {"options": "-vn", "before_options": "-reconnect 1 -reconnect_streamed 1 -reconnect_delay_max 0 -nostdin"}
        else:
            self.ffmpeg_opts = {"options": "-vn", "before_options": "-nostdin"}
    def disable(self):
        self.music.players.remove(self)
    def on_queue(self, func):
        self.on_queue_func = func
    def on_play(self, func):
        self.on_play_func = func
    def on_skip(self, func):
        self.on_skip_func = func
    def on_stop(self, func):
        self.on_stop_func = func
    def on_pause(self, func):
        self.on_pause_func = func
    def on_resume(self, func):
        self.on_resume_func = func
    def on_loop_toggle(self, func):
        self.on_loop_toggle_func = func
    def on_volume_change(self, func):
        self.on_volume_change_func = func
    def on_remove_from_queue(self, func):
        self.on_remove_from_queue_func = func
    async def queue(self, url, search=False, bettersearch=False):
        song = await get_video_data(url, search, bettersearch, self.loop)
        self.music.queue[self.ctx.guild.id].append(song)
        if self.on_queue_func:
            await self.on_queue_func(self.ctx, song)
        return song
    async def play(self):
        source = discord.PCMVolumeTransformer(discord.FFmpegPCMAudio(self.music.queue[self.ctx.guild.id][0].source, **self.ffmpeg_opts))
        self.voice.play(source, after=lambda error: self.after_func(self.ctx, self.ffmpeg_opts, self.music, self.after_func, self.on_play_func, self.loop))
        song = self.music.queue[self.ctx.guild.id][0]
        if self.on_play_func:
            await self.on_play_func(self.ctx, song)
        return song
    async def skip(self, force=False):
        if len(self.music.queue[self.ctx.guild.id]) == 0:
            raise NotPlaying("Cannot loop because nothing is being played")
        elif not len(self.music.queue[self.ctx.guild.id]) > 1 and not force:
            raise EmptyQueue("Cannot skip because queue is empty")
        else:
            old = self.music.queue[self.ctx.guild.id][0]
            old.is_looping = False if old.is_looping else False
            self.voice.stop()
            try:
                new = self.music.queue[self.ctx.guild.id][0]
                if self.on_skip_func:
                    await self.on_skip_func(self.ctx, old, new)
                return (old, new)
            except IndexError:
                if self.on_skip_func:
                    await self.on_skip_func(self.ctx, old)
                return old        
    async def stop(self):
        try:
            self.music.queue[self.ctx.guild.id] = []
            self.voice.stop()
            self.music.players.remove(self)
        except:
            raise NotPlaying("Cannot loop because nothing is being played")
        if self.on_stop_func:
            await self.on_stop_func(self.ctx)
    async def pause(self):
        try:
            self.voice.pause()
            song = self.music.queue[self.ctx.guild.id][0]
        except:
            raise NotPlaying("Cannot pause because nothing is being played")
        if self.on_pause_func:
            await self.on_pause_func(self.ctx, song)
        return song
    async def resume(self):
        try:
            self.voice.resume()
            song = self.music.queue[self.ctx.guild.id][0]
        except:
            raise NotPlaying("Cannot resume because nothing is being played")
        if self.on_resume_func:
            await self.on_resume_func(self.ctx, song)
        return song
    def current_queue(self):
        try:
            return self.music.queue[self.ctx.guild.id]
        except KeyError:
            raise EmptyQueue("Queue is empty")
    def now_playing(self):
        try:
            return self.music.queue[self.ctx.guild.id][0]
        except:
            return None
    async def toggle_song_loop(self):
        try:
            song = self.music.queue[self.ctx.guild.id][0]
        except:
            raise NotPlaying("Cannot loop because nothing is being played")
        if not song.is_looping:
            song.is_looping = True
        else:
            song.is_looping = False
        if self.on_loop_toggle_func:
            await self.on_loop_toggle_func(self.ctx, song)
        return song
    async def change_volume(self, vol):
        self.voice.source.volume = vol
        try:
            song = self.music.queue[self.ctx.guild.id][0]
        except:
            raise NotPlaying("Cannot loop because nothing is being played")
        if self.on_volume_change_func:
            await self.on_volume_change_func(self.ctx, song, vol)
        return (song, vol)
    async def remove_from_queue(self, index):
        if index == 0:
            try:
                song = self.music.queue[self.ctx.guild.id][0]
            except:
                raise NotPlaying("Cannot loop because nothing is being played")
            await self.skip(force=True)
            return song
        song = self.music.queue[self.ctx.guild.id][index]
        self.music.queue[self.ctx.guild.id].pop(index)
        if self.on_remove_from_queue_func:
            await self.on_remove_from_queue_func(self.ctx, song)
        return song
    def delete(self):
        self.music.players.remove(self)
        
class Song(object):
    def __init__(self, source, url, title, description, views, duration, thumbnail, channel, channel_url, loop):
        self.source = source
        self.url = url
        self.title = title
        self.description = description
        self.views = views
        self.name = title
        self.duration = duration
        self.thumbnail = thumbnail
        self.channel = channel
        self.channel_url = channel_url
        self.is_looping = loop



import setuptools

with open("README.md", "r", encoding="utf-8", errors="ignore") as fh:
    long_description = fh.read()

setuptools.setup(
    name="new-DisUtils",
    version="1.3.5",
    author="DTS",
    description="Dis-Utils is a very useful library made to be used with pycord.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.github.com/DTS-11/Dis-Utils",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">= 3.6",
    include_package_data=True,
    install_requires=["py-cord"],
    extras_require={"voice": ["py-cord[voice]", "youtube-dl"]}
)
